import logging
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.backends import default_backend

logger = logging.getLogger(__name__)


def encrypt(key, plain_data, mode='ecb', iv='00' * 16):  # key/plain_data/iv可以传入16进制字符串或者bytes
    if not all(isinstance(i, (bytes, str)) for i in (key, plain_data, iv)):
        raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
    try:
        plain_data = plain_data if isinstance(plain_data, bytes) else bytes.fromhex(plain_data)
        key = key if isinstance(key, bytes) else bytes.fromhex(key)
        iv = iv if isinstance(iv, bytes) else bytes.fromhex(iv)
    except ValueError:
        raise ValueError("不是合法的十六进制字符串，无法转换为bytes")
    mode = mode.lower()
    if mode == 'cbc':
        mode = modes.CBC(iv)
    elif mode == 'ecb':
        mode = modes.ECB()
    else:
        raise ValueError("不支持的模式,当前sm4只支持cbc/ecb模式")
    # 使用PKCS7进行填充
    pad = padding.PKCS7(algorithms.SM4.block_size).padder()
    padded_plain_data = pad.update(plain_data) + pad.finalize()
    # 创建SM4对象
    cipher = Cipher(algorithms.SM4(key), mode, backend=default_backend())
    encryptor = cipher.encryptor()  # 加密
    logger.debug(f"开始进行sm4加密计算,算法模式{mode.name},加密密钥:{key.hex()},加密数据长度:{len(plain_data) / 2}字节")
    cipher_data = encryptor.update(padded_plain_data)
    logger.debug(f"完成sm4加密计算")
    return cipher_data.hex()


def decrypt(key, cipher_data, mode='ecb', iv='00' * 16):  # key/cipher_data/iv可以传入16进制字符串或者bytes
    if not all(isinstance(i, (bytes, str)) for i in (key, cipher_data, iv)):
        raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
    try:
        cipher_data = cipher_data if isinstance(cipher_data, bytes) else bytes.fromhex(cipher_data)
        key = key if isinstance(key, bytes) else bytes.fromhex(key)
        iv = iv if isinstance(iv, bytes) else bytes.fromhex(iv)
    except ValueError:
        raise ValueError("不是合法的十六进制字符串，无法转换为bytes")
    mode = mode.lower()
    if mode == 'cbc':
        mode = modes.CBC(iv)
    elif mode == 'ecb':
        mode = modes.ECB()
    else:
        raise ValueError("不支持的模式,当前sm4只支持cbc/ecb模式")
    # 创建SM4对象
    cipher = Cipher(algorithms.SM4(key), mode, backend=default_backend())
    decryptor = cipher.decryptor()  # 解密
    logger.debug(f"开始进行sm4解密计算,算法模式{mode.name},解密密钥:{key.hex()}")
    padded_plain_data = decryptor.update(cipher_data)
    # 去除填充
    unpad = padding.PKCS7(algorithms.SM4.block_size).unpadder()
    plain_data = unpad.update(padded_plain_data) + unpad.finalize()
    logger.debug(f"完成sm4解密计算,解密后的数据长度:{len(plain_data)}字节")
    return plain_data.hex()



import secrets
import pytest
from gd_sm import *

"""
对gd_sm库进行测试，cd test进行本目录，执行pytest test.py
"""


class TestSM2:
    def setup_class(self):
        pass

    def test_01(self):
        """ 测试对16进制类型的数据、bytes类型的数据进行签名和验签 """
        m2 = sm2.SM2()
        test_data_hex = secrets.token_hex(128)
        test_data_bytes = bytes.fromhex(test_data_hex)
        sign_data1 = m2.sign(test_data_hex)
        sign_data2 = m2.sign(test_data_bytes)
        sign_data_bytes = bytes.fromhex(sign_data2)
        assert m2.verify(test_data_hex, sign_data1)
        assert m2.verify(test_data_bytes, sign_data1)
        assert m2.verify(test_data_bytes, sign_data_bytes)
        assert m2.verify(test_data_hex, sign_data_bytes)

    def test_02(self):
        """ 测试对16进制类型的数据、bytes类型的数据进行加密和解密 """
        m2 = sm2.SM2()
        test_data_hex = secrets.token_hex(128)
        test_data_bytes = bytes.fromhex(test_data_hex)
        d1 = m2.encrypt(test_data_hex)
        d2 = m2.encrypt(test_data_bytes)
        d2_bytes = bytes.fromhex(d2)
        assert m2.decrypt(d1) == test_data_hex
        assert m2.decrypt(d2_bytes) == test_data_hex

    def test_03(self):
        """ 测试传入的数据必须是16进制字符串或者bytes类型 """
        m2 = sm2.SM2()
        with pytest.raises(TypeError):
            m2.sign(123123)
            m2.sign([1, 2, 3])
            m2.sign("zzzz")


class TestSM3:
    def setup_class(self):
        pass

    def test_01(self):
        """ 测试对16进制类型的数据、bytes类型的数据进行hash计算和验证 """
        test_data_hex = secrets.token_hex(128)
        test_data_bytes = bytes.fromhex(test_data_hex)
        h1 = sm3.sm3_hash(test_data_hex)
        h2 = sm3.sm3_hash(test_data_bytes)
        h2_bytes = bytes.fromhex(h2)
        assert sm3.check_sm3(test_data_hex, h1)
        assert sm3.check_sm3(test_data_hex, h2_bytes)
        assert sm3.check_sm3(test_data_bytes, h2_bytes)
        assert sm3.check_sm3(test_data_bytes, h1)

    def test_02(self):
        """ 测试传入的数据必须是16进制字符串或者bytes类型 """
        with pytest.raises(TypeError):
            sm3.sm3_hash(123123)
            sm3.sm3_hash([1, 2, 3])
            sm3.sm3_hash("zzzz")
        with pytest.raises(TypeError):
            sm3.check_sm3(123123, "11")
            sm3.check_sm3([1, 2, 3], "11")
            sm3.check_sm3("zzzz", "11")
            sm3.check_sm3("11", 123123)
            sm3.check_sm3("11", [1, 2, 3])
            sm3.check_sm3("11", "zzzz")


class TestSM4:
    def setup_class(self):
        pass

    def test_01(self):
        """ 测试对16进制类型的数据、bytes类型的数据进行加密和解密验证（cbc模式） """
        test_data_hex = secrets.token_hex(128)
        test_data_bytes = bytes.fromhex(test_data_hex)
        key = secrets.token_hex(16)
        key_bytes = bytes.fromhex(key)
        h1 = sm4.encrypt(key, test_data_hex)
        h2 = sm4.encrypt(key_bytes, test_data_bytes)
        h2_bytes = bytes.fromhex(h2)
        assert sm4.decrypt(key, h1) == test_data_hex
        assert sm4.decrypt(key_bytes, h2_bytes) == test_data_hex

    def test_02(self):
        """ 测试cbc模式加密 """
        test_data = secrets.token_hex(128)
        key = secrets.token_hex(16)
        iv = secrets.token_hex(16)
        enc_data = sm4.encrypt(key=key, plain_data=test_data, mode="cbc", iv=iv)
        assert sm4.decrypt(key, enc_data, mode="cbc", iv=iv) == test_data

    def test_03(self):
        """ 测试加密传入的密钥key，iv和数据必须是16进制字符串或者bytes类型 """
        test_data = secrets.token_hex(128)
        key = secrets.token_hex(16)
        iv = secrets.token_hex(16)
        with pytest.raises(TypeError):
            key = secrets.token_hex(16)
            sm4.encrypt(1231233, test_data)
            sm4.encrypt([1, 3, 4], test_data)
            sm4.encrypt("123", test_data)
            sm4.encrypt(key, 1231233)
            sm4.encrypt(key, [1, 3, 4])
            sm4.encrypt(key, "123")
            sm4.encrypt(key, test_data, mode="cbc", iv=1231233)
            sm4.encrypt(key, test_data, mode="cbc", iv=[1, 3, 4])
            sm4.encrypt(key, test_data, mode="cbc", iv="123")

    def test_04(self):
        """ 测试解密传入的密钥key，iv和数据必须是16进制字符串或者bytes类型 """
        test_data = secrets.token_hex(128)
        key = secrets.token_hex(16)
        iv = secrets.token_hex(16)
        with pytest.raises(TypeError):
            key = secrets.token_hex(16)
            sm4.decrypt(1231233, test_data)
            sm4.decrypt([1, 3, 4], test_data)
            sm4.decrypt("123", test_data)
            sm4.decrypt(key, 1231233)
            sm4.decrypt(key, [1, 3, 4])
            sm4.decrypt(key, "123")
            sm4.decrypt(key, test_data, mode="cbc", iv=1231233)
            sm4.decrypt(key, test_data, mode="cbc", iv=[1, 3, 4])
            sm4.decrypt(key, test_data, mode="cbc", iv="123")

    def test_05(self):
        """ 测试加密解密只支持ecb/cbc模式 """
        test_data = secrets.token_hex(128)
        key = secrets.token_hex(16)
        enc_data = sm4.encrypt(key, test_data, mode="Ecb")
        sm4.decrypt(key, enc_data, mode="EcB")
        enc_data = sm4.encrypt(key, test_data, mode="CbC")
        sm4.decrypt(key, enc_data, mode="cbC")
        with pytest.raises(ValueError):
            sm4.encrypt(key, test_data, mode="test")
            sm4.encrypt(key, test_data, mode="c bc")
            sm4.decrypt(key, test_data, mode=["c", "b", "c"])
            sm4.decrypt(key, test_data, mode="cb c")

    def test_06(self):
        """ 测试pkcs7填充"""
        key = secrets.token_hex(16)
        for i in range(16):
            test_data = secrets.token_hex(128 + i)
            enc_data = sm4.encrypt(key, test_data)
            data = sm4.decrypt(key, enc_data)
            assert data == test_data
            assert len(enc_data) == 288



from . import _sm2
import logging

logger = logging.getLogger(__name__)


class SM2(_sm2.CryptSM2):

    def __init__(self, private_key=None):
        super().__init__(private_key)

    def sign(self, data):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        if not isinstance(data, (str, bytes)):
            raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
        return self._sign_with_sm3(data)

    def verify(self, data, sign):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        if not isinstance(data, (str, bytes)):
            raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
        return self._verify_with_sm3(sign, data)

    def encrypt(self, data):
        if not isinstance(data, (str, bytes)):
            raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
        return self._encrypt(data)  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理

    def decrypt(self, data):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        if not isinstance(data, (str, bytes)):
            raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
        return self._decrypt(data)



from gd_sm import *

# sm2使用示例
s2 = sm2.SM2()  # 示例化一个sm2对象
print(f"实例化一个sm2对象，私钥:{s2.private_key}, 公钥:{s2.public_key}")  # 打印对象的公私钥
data = "C27AA65FCA082F8959D8B0F71764B6C5CA02C0AAD27C0CF6B393E61136B82EA9"  # 待处理的数据,16进制字符串格式
sign_data = s2.sign(data)  # 使用私钥签名
valid_sign = s2.verify(data, sign_data)  # 公钥验签
assert valid_sign is True
enc_data = s2.encrypt(data)  # 公钥加密
dec_data = s2.decrypt(enc_data)  # 私钥解密
assert dec_data.lower() == data.lower()

# sm3使用示例
data = "C27AA65FCA082F8959D8B0F71764B6C5CA02C0AAD27C0CF6B393E61136B82EA9"  # 待处理的数据,16进制字符串格式
data_hash = sm3.sm3_hash(data)
valid_hash = sm3.check_sm3(data, data_hash)
assert valid_hash is True

# sm4使用示例
data = "C27AA65FCA082F8959D8B0F71764B6C5CA02C0AAD27C0CF6B393E61136B82EA9"  # 待处理的数据,16进制字符串格式
key = "000102030405060708090A0b0c0d0e0f"
iv = "00000101020203030404050506060707"
enc_data = sm4.encrypt(key, data)  # 默认ecb模式，pkcs7 padding
dec_data = sm4.decrypt(key, enc_data)
assert dec_data.lower() == data.lower()
enc_data = sm4.encrypt(key, data, mode="cbc", iv=iv)  # cbc模式，pkcs7 padding
dec_data = sm4.decrypt(key, enc_data, mode="cbc", iv=iv)
assert dec_data.lower() == data.lower()



from cryptography.hazmat.primitives import hashes
from math import ceil
import binascii


def sm3_hash(msg):  # msg可以是16进制字符串或者bytes
    if not isinstance(msg, (str, bytes)):
        raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
    try:
        msg = msg if isinstance(msg, bytes) else bytes.fromhex(msg)
    except ValueError:
        raise ValueError("不是合法的十六进制字符串，无法转换为bytes")
    d = hashes.Hash(hashes.SM3())
    d.update(msg)
    value = d.finalize()
    return value.hex()


def check_sm3(msg, hash_data):  # msg/hash_data可以是16进制字符串或者bytes
    if not isinstance(msg, (str, bytes)):
        raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
    if not isinstance(hash_data, (str, bytes)):
        raise TypeError("传入了非法的数据类型，传入的数据必须是16进制字符串或者bytes类型")
    try:
        msg = msg if isinstance(msg, str) else msg.hex()
        hash_data = hash_data if isinstance(hash_data, str) else hash_data.hex()
    except ValueError:
        raise ValueError("不是合法的十六进制字符串，无法转换为bytes")
    return True if sm3_hash(msg) == hash_data else False


def sm3_kdf(z, key_len):  # z为16进制字符串（str），key_len为密钥长度（单位byte），这个方法是sm2算法使用
    key_len = int(key_len)
    ct = 0x00000001  # 计数器，标准要求不能超过2的32次方
    count = ceil(key_len / 32)  # 迭代次数
    zin = [i for i in bytes.fromhex(z.decode('utf8'))]
    ha = ""
    for i in range(count):
        msg = zin + [i for i in binascii.a2b_hex(('%08x' % ct).encode('utf8'))]
        ha = ha + sm3_hash(bytes(msg))
        ct += 1
    return ha[0: key_len * 2]



from . import sm3
import logging
import secrets

logger = logging.getLogger(__name__)


class CryptSM2(object):
    ecc_table = {  # 选择素域，设置椭圆曲线参数
        'n': 'FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123',
        'p': 'FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFF',
        'g': '32c4ae2c1f1981195f9904466a39c9948fe30bbff2660be1715a4589334c74c7'
             'bc3736a2f4f6779c59bdcee36b692153d0a9877cc62a474002df32e52139f0a0',
        'a': 'FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FFFFFFFFFFFFFFFC',
        'b': '28E9FA9E9D9F5E344D5A9E4BCF6509A7F39789F515AB8F92DDBCBD414D940E93',
    }

    @staticmethod
    def generate_private_key():  # 16进制字符串格式
        while True:
            key = secrets.token_hex(32)
            if CryptSM2.is_valid_private_key(key):
                logger.debug(f"生成符合sm2的[1, n-1]区间数据:{key}")
                return key

    @staticmethod
    def is_valid_private_key(d):  # 检查私钥或者过程中的k是否满足在[1, n-1]区间内
        n = int('FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123', 16)
        number = int(d, 16)
        return 1 <= number <= n - 1

    def __init__(self, private_key=None):
        self.private_key = private_key if private_key is not None else self.generate_private_key()
        logger.debug(f"SM2对象的私钥:{self.private_key}")
        self.para_len = len(self.ecc_table['n'])
        self.ecc_a3 = (int(self.ecc_table['a'], base=16) + 3) % int(self.ecc_table['p'], base=16)
        self.ecc_table = CryptSM2.ecc_table
        self.public_key = self.private_key_export_pubkey()
        logger.debug(f"SM2对象的公钥:{self.public_key}")

    def private_key_export_pubkey(self, private_key=None):
        if private_key is None:
            private_key = self.private_key
        pubkey = self._kg(int(private_key, 16), self.ecc_table['g'])
        logger.debug(f"通过私钥{private_key}导出sm2公钥{pubkey}")
        return pubkey

    def _kg(self, k, point):  # kP运算
        point = '%s%s' % (point, '1')
        mask_str = '8'
        for i in range(self.para_len - 1):
            mask_str += '0'
        mask = int(mask_str, 16)
        temp = point
        flag = False
        for n in range(self.para_len * 4):
            if flag and isinstance(temp, str):
                temp = self._double_point(temp)
            if (k & mask) != 0:
                if flag:
                    temp = self._add_point(temp, point)
                else:
                    flag = True
                    temp = point
            k = k << 1
        return self._convert_jacb_to_nor(temp)

    def _double_point(self, point):  # 倍点
        try:
            l_p = len(point)
        except Exception as e:
            raise ValueError(f"传入的point非法:{e}")
        len_2 = 2 * self.para_len
        if l_p < self.para_len * 2:
            return None
        else:
            x1 = int(point[0:self.para_len], 16)
            y1 = int(point[self.para_len:len_2], 16)
            if l_p == len_2:
                z1 = 1
            else:
                z1 = int(point[len_2:], 16)
            t6 = (z1 * z1) % int(self.ecc_table['p'], base=16)
            t2 = (y1 * y1) % int(self.ecc_table['p'], base=16)
            t3 = (x1 + t6) % int(self.ecc_table['p'], base=16)
            t4 = (x1 - t6) % int(self.ecc_table['p'], base=16)
            t1 = (t3 * t4) % int(self.ecc_table['p'], base=16)
            t3 = (y1 * z1) % int(self.ecc_table['p'], base=16)
            t4 = (t2 * 8) % int(self.ecc_table['p'], base=16)
            t5 = (x1 * t4) % int(self.ecc_table['p'], base=16)
            t1 = (t1 * 3) % int(self.ecc_table['p'], base=16)
            t6 = (t6 * t6) % int(self.ecc_table['p'], base=16)
            t6 = (self.ecc_a3 * t6) % int(self.ecc_table['p'], base=16)
            t1 = (t1 + t6) % int(self.ecc_table['p'], base=16)
            z3 = (t3 + t3) % int(self.ecc_table['p'], base=16)
            t3 = (t1 * t1) % int(self.ecc_table['p'], base=16)
            t2 = (t2 * t4) % int(self.ecc_table['p'], base=16)
            x3 = (t3 - t5) % int(self.ecc_table['p'], base=16)
            if (t5 % 2) == 1:
                t4 = (t5 + ((t5 + int(self.ecc_table['p'], base=16)) >> 1) - t3) % int(
                    self.ecc_table['p'], base=16)
            else:
                t4 = (t5 + (t5 >> 1) - t3) % int(self.ecc_table['p'], base=16)
            t1 = (t1 * t4) % int(self.ecc_table['p'], base=16)
            y3 = (t1 - t2) % int(self.ecc_table['p'], base=16)
            form = '%%0%dx' % self.para_len
            form = form * 3
            return form % (x3, y3, z3)

    def _add_point(self, p1, p2):  # 点加函数，P2点为仿射坐标即z=1，P1为Jacobi加重射影坐标
        len_2 = 2 * self.para_len
        l1 = len(p1)
        l2 = len(p2)
        if (l1 < len_2) or (l2 < len_2):
            return None
        else:
            x1 = int(p1[0:self.para_len], 16)
            y1 = int(p1[self.para_len:len_2], 16)
            if l1 == len_2:
                z1 = 1
            else:
                z1 = int(p1[len_2:], 16)
            x2 = int(p2[0:self.para_len], 16)
            y2 = int(p2[self.para_len:len_2], 16)
            t1 = (z1 * z1) % int(self.ecc_table['p'], base=16)
            t2 = (y2 * z1) % int(self.ecc_table['p'], base=16)
            t3 = (x2 * t1) % int(self.ecc_table['p'], base=16)
            t1 = (t1 * t2) % int(self.ecc_table['p'], base=16)
            t2 = (t3 - x1) % int(self.ecc_table['p'], base=16)
            t3 = (t3 + x1) % int(self.ecc_table['p'], base=16)
            t4 = (t2 * t2) % int(self.ecc_table['p'], base=16)
            t1 = (t1 - y1) % int(self.ecc_table['p'], base=16)
            z3 = (z1 * t2) % int(self.ecc_table['p'], base=16)
            t2 = (t2 * t4) % int(self.ecc_table['p'], base=16)
            t3 = (t3 * t4) % int(self.ecc_table['p'], base=16)
            t5 = (t1 * t1) % int(self.ecc_table['p'], base=16)
            t4 = (x1 * t4) % int(self.ecc_table['p'], base=16)
            x3 = (t5 - t3) % int(self.ecc_table['p'], base=16)
            t2 = (y1 * t2) % int(self.ecc_table['p'], base=16)
            t3 = (t4 - x3) % int(self.ecc_table['p'], base=16)
            t1 = (t1 * t3) % int(self.ecc_table['p'], base=16)
            y3 = (t1 - t2) % int(self.ecc_table['p'], base=16)
            form = '%%0%dx' % self.para_len
            form = form * 3
            return form % (x3, y3, z3)

    def _convert_jacb_to_nor(self, point):  # Jacobi加重射影坐标转换成仿射坐标
        len_2 = 2 * self.para_len
        x = int(point[0:self.para_len], 16)
        y = int(point[self.para_len:len_2], 16)
        z = int(point[len_2:], 16)
        z_inv = pow(z, int(self.ecc_table['p'], base=16) - 2, int(self.ecc_table['p'], base=16))
        z_inv_squar = (z_inv * z_inv) % int(self.ecc_table['p'], base=16)
        z_inv_qube = (z_inv_squar * z_inv) % int(self.ecc_table['p'], base=16)
        x_new = (x * z_inv_squar) % int(self.ecc_table['p'], base=16)
        y_new = (y * z_inv_qube) % int(self.ecc_table['p'], base=16)
        z_new = (z * z_inv) % int(self.ecc_table['p'], base=16)
        if z_new == 1:
            form = '%%0%dx' % self.para_len
            form = form * 2
            return form % (x_new, y_new)
        else:
            return None

    def _verify(self, sign, data):  # 签名sign和数据data都是16进制字符串格式
        # 验签函数，sign签名r||s，E消息hash，public_key公钥
        r = int(sign[0:self.para_len], 16)
        s = int(sign[self.para_len:2 * self.para_len], 16)
        e = int(data, 16)
        t = (r + s) % int(self.ecc_table['n'], base=16)
        if t == 0:
            return 0
        p1 = self._kg(s, self.ecc_table['g'])
        p2 = self._kg(t, self.public_key)
        if p1 == p2:
            p1 = '%s%s' % (p1, 1)
            p1 = self._double_point(p1)
        else:
            p1 = '%s%s' % (p1, 1)
            p1 = self._add_point(p1, p2)
            p1 = self._convert_jacb_to_nor(p1)
        x = int(p1[0:self.para_len], 16)
        return r == ((e + x) % int(self.ecc_table['n'], base=16))

    def _sign(self, data, k):  # data/k 均为16进制字符串
        e = int(data, 16)  # 消息转化为16进制字符串
        d = int(self.private_key, 16)
        k = int(k, 16)
        p1 = self._kg(k, self.ecc_table['g'])
        x = int(p1[0:self.para_len], 16)
        r = ((e + x) % int(self.ecc_table['n'], base=16))
        if r == 0 or r + k == int(self.ecc_table['n'], base=16):
            return None
        d_1 = pow(
            d + 1, int(self.ecc_table['n'], base=16) - 2, int(self.ecc_table['n'], base=16))
        s = (d_1 * (k + r) - r) % int(self.ecc_table['n'], base=16)
        if s == 0:
            return None
        else:
            return '%064x%064x' % (r, s)

    def _encrypt(self, data):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        # 加密函数，data消息(bytes)
        if isinstance(data, bytes):
            msg = data.hex()  # 消息转化为16进制字符串
        else:
            msg = data
        k = self.generate_private_key()
        c1 = self._kg(int(k, 16), self.ecc_table['g'])
        xy = self._kg(int(k, 16), self.public_key)
        x2 = xy[0:self.para_len]
        y2 = xy[self.para_len:2 * self.para_len]
        ml = len(msg)
        t = sm3.sm3_kdf(xy.encode('utf8'), ml / 2)
        if int(t, 16) == 0:
            return None
        else:
            form = '%%0%dx' % ml
            c2 = form % (int(msg, 16) ^ int(t, 16))
            c3 = sm3.sm3_hash(bytes.fromhex(x2 + msg + y2))
            return c1 + c3 + c2

    def _decrypt(self, data):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        # 解密函数，data密文（bytes）
        if isinstance(data, bytes):
            data = data.hex()  # 消息转化为16进制字符串
        len_2 = 2 * self.para_len
        len_3 = len_2 + 64
        c1 = data[0:len_2]
        c3 = data[len_2:len_3]
        c2 = data[len_3:]
        xy = self._kg(int(self.private_key, 16), c1)
        x2 = xy[0:self.para_len]
        y2 = xy[self.para_len:len_2]
        cl = len(c2)
        t = sm3.sm3_kdf(xy.encode('utf8'), cl / 2)
        if int(t, 16) == 0:
            return None
        else:
            form = '%%0%dx' % cl
            M = form % (int(c2, 16) ^ int(t, 16))
            u = sm3.sm3_hash(bytes.fromhex(x2 + M + y2))
            return M

    def _sm3_z(self, data):
        z = '0080' + '31323334353637383132333435363738' + \
            self.ecc_table['a'] + self.ecc_table['b'] + self.ecc_table['g'] + self.public_key
        za = sm3.sm3_hash(z)
        m_ = za + data
        e = sm3.sm3_hash(m_)
        return e

    def _sign_with_sm3(self, data):  # data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        if isinstance(data, bytes):
            data = data.hex()
        sign_data = self._sm3_z(data)
        random_hex_str = self.generate_private_key()
        sign = self._sign(sign_data, random_hex_str)  # 16进制
        return sign

    def _verify_with_sm3(self, sign, data):  # sign、data可以是16进制字符串或者bytes，统一转成16进制字符串处理
        if isinstance(sign, bytes):
            sign = sign.hex()
        if isinstance(data, bytes):
            data = data.hex()
        sign_data = self._sm3_z(data)
        return self._verify(sign, sign_data)
